import asyncio
from typing import Any, Coroutine

def run_async_coro(
    tasks: list[Coroutine] | Coroutine,
    times: int | None = None
) -> Any | Coroutine:
    
    # run the tasks in parallel
    async def gather_async(*tasks):
        return await asyncio.gather(*tasks)
    
    async def single_task(task):
        if asyncio.iscoroutine(task):
            return await task
        else:
            return task
    
    async def run_multiple_times(coro, times):
        return await asyncio.gather(*[
            coro
            for _ in range(times)
        ])
    
    if not isinstance(tasks, (list, tuple)):
        coroutine = single_task(tasks)
    else:
        coroutine = gather_async(*tasks)
    
    if times is not None:
        coroutine = run_multiple_times(coroutine, times)
    
    # if we are in a running event loop, return coroutine to be awaited
    if asyncio.get_event_loop().is_running():
        return coroutine
    
    return asyncio.run(coroutine)

def run_async(coro):
    """
    Run a coroutine in a synchronous context by creating a new event loop.

    This function is safe to use when no other asyncio loop is running.
    
    Args:
        coro: A coroutine object to run.

    Returns:
        The result of the coroutine.
    """
    # Create a new event loop.
    loop = asyncio.new_event_loop()
    try:
        # Set the new event loop as the current loop.
        asyncio.set_event_loop(loop)
        # Run the coroutine until it completes and return its result.
        return loop.run_until_complete(coro)
    finally:
        # Close the loop to free up resources.
        loop.close()
        # Reset the current event loop to prevent any potential memory issues.
        asyncio.set_event_loop(None)


"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from honeyhive.models.components import metric as components_metric
from honeyhive.types import BaseModel
from honeyhive.utils import FieldMetadata, QueryParamMetadata
import httpx
from typing import List, Optional, TypedDict
from typing_extensions import Annotated, NotRequired


class GetMetricsRequestTypedDict(TypedDict):
    project_name: str
    r"""Project name associated with metrics"""


class GetMetricsRequest(BaseModel):
    project_name: Annotated[
        str, FieldMetadata(query=QueryParamMetadata(style="form", explode=True))
    ]
    r"""Project name associated with metrics"""


class GetMetricsResponseTypedDict(TypedDict):
    content_type: str
    r"""HTTP response content type for this operation"""
    status_code: int
    r"""HTTP response status code for this operation"""
    raw_response: httpx.Response
    r"""Raw HTTP response; suitable for custom response parsing"""
    metrics: NotRequired[List[components_metric.MetricTypedDict]]
    r"""A list of metrics"""


class GetMetricsResponse(BaseModel):
    content_type: str
    r"""HTTP response content type for this operation"""

    status_code: int
    r"""HTTP response status code for this operation"""

    raw_response: httpx.Response
    r"""Raw HTTP response; suitable for custom response parsing"""

    metrics: Optional[List[components_metric.Metric]] = None
    r"""A list of metrics"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from honeyhive.models.components import project as components_project
from honeyhive.types import BaseModel
from honeyhive.utils import FieldMetadata, QueryParamMetadata
import httpx
from typing import List, Optional, TypedDict
from typing_extensions import Annotated, NotRequired


class GetProjectsRequestTypedDict(TypedDict):
    name: NotRequired[str]


class GetProjectsRequest(BaseModel):
    name: Annotated[
        Optional[str],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None


class GetProjectsResponseTypedDict(TypedDict):
    content_type: str
    r"""HTTP response content type for this operation"""
    status_code: int
    r"""HTTP response status code for this operation"""
    raw_response: httpx.Response
    r"""Raw HTTP response; suitable for custom response parsing"""
    projects: NotRequired[List[components_project.ProjectTypedDict]]
    r"""A list of projects"""


class GetProjectsResponse(BaseModel):
    content_type: str
    r"""HTTP response content type for this operation"""

    status_code: int
    r"""HTTP response status code for this operation"""

    raw_response: httpx.Response
    r"""Raw HTTP response; suitable for custom response parsing"""

    projects: Optional[List[components_project.Project]] = None
    r"""A list of projects"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from honeyhive.types import BaseModel
import httpx
import pydantic
from typing import Optional, TypedDict
from typing_extensions import Annotated, NotRequired


class ResultTypedDict(TypedDict):
    inserted_id: NotRequired[str]


class Result(BaseModel):
    inserted_id: Annotated[Optional[str], pydantic.Field(alias="insertedId")] = None


class CreateToolResponseBodyTypedDict(TypedDict):
    r"""Tool successfully created"""

    result: NotRequired[ResultTypedDict]


class CreateToolResponseBody(BaseModel):
    r"""Tool successfully created"""

    result: Optional[Result] = None


class CreateToolResponseTypedDict(TypedDict):
    content_type: str
    r"""HTTP response content type for this operation"""
    status_code: int
    r"""HTTP response status code for this operation"""
    raw_response: httpx.Response
    r"""Raw HTTP response; suitable for custom response parsing"""
    object: NotRequired[CreateToolResponseBodyTypedDict]
    r"""Tool successfully created"""


class CreateToolResponse(BaseModel):
    content_type: str
    r"""HTTP response content type for this operation"""

    status_code: int
    r"""HTTP response status code for this operation"""

    raw_response: httpx.Response
    r"""Raw HTTP response; suitable for custom response parsing"""

    object: Optional[CreateToolResponseBody] = None
    r"""Tool successfully created"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from honeyhive.types import BaseModel
from typing import Optional, TypedDict
from typing_extensions import NotRequired


class CreateProjectRequestTypedDict(TypedDict):
    name: str
    description: NotRequired[str]


class CreateProjectRequest(BaseModel):
    name: str

    description: Optional[str] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from honeyhive.models.components import (
    updaterunrequest as components_updaterunrequest,
    updaterunresponse as components_updaterunresponse,
)
from honeyhive.types import BaseModel
from honeyhive.utils import FieldMetadata, PathParamMetadata, RequestMetadata
import httpx
from typing import Optional, TypedDict
from typing_extensions import Annotated, NotRequired


class UpdateRunRequestTypedDict(TypedDict):
    run_id: str
    update_run_request: components_updaterunrequest.UpdateRunRequestTypedDict


class UpdateRunRequest(BaseModel):
    run_id: Annotated[
        str, FieldMetadata(path=PathParamMetadata(style="simple", explode=False))
    ]

    update_run_request: Annotated[
        components_updaterunrequest.UpdateRunRequest,
        FieldMetadata(request=RequestMetadata(media_type="application/json")),
    ]


class UpdateRunResponseTypedDict(TypedDict):
    content_type: str
    r"""HTTP response content type for this operation"""
    status_code: int
    r"""HTTP response status code for this operation"""
    raw_response: httpx.Response
    r"""Raw HTTP response; suitable for custom response parsing"""
    update_run_response: NotRequired[
        components_updaterunresponse.UpdateRunResponseTypedDict
    ]
    r"""Successful response"""


class UpdateRunResponse(BaseModel):
    content_type: str
    r"""HTTP response content type for this operation"""

    status_code: int
    r"""HTTP response status code for this operation"""

    raw_response: httpx.Response
    r"""Raw HTTP response; suitable for custom response parsing"""

    update_run_response: Optional[components_updaterunresponse.UpdateRunResponse] = None
    r"""Successful response"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum
from honeyhive.models.components import configuration as components_configuration
from honeyhive.types import BaseModel
from honeyhive.utils import FieldMetadata, QueryParamMetadata
import httpx
from typing import List, Optional, TypedDict
from typing_extensions import Annotated, NotRequired


class Env(str, Enum):
    r"""Environment - \"dev\", \"staging\" or \"prod\" """

    DEV = "dev"
    STAGING = "staging"
    PROD = "prod"


class GetConfigurationsRequestTypedDict(TypedDict):
    project: str
    r"""Project name for configuration like `Example Project`"""
    env: NotRequired[Env]
    r"""Environment - \"dev\", \"staging\" or \"prod\" """
    name: NotRequired[str]
    r"""The name of the configuration like `v0`"""


class GetConfigurationsRequest(BaseModel):
    project: Annotated[
        str, FieldMetadata(query=QueryParamMetadata(style="form", explode=True))
    ]
    r"""Project name for configuration like `Example Project`"""

    env: Annotated[
        Optional[Env],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None
    r"""Environment - \"dev\", \"staging\" or \"prod\" """

    name: Annotated[
        Optional[str],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None
    r"""The name of the configuration like `v0`"""


class GetConfigurationsResponseTypedDict(TypedDict):
    content_type: str
    r"""HTTP response content type for this operation"""
    status_code: int
    r"""HTTP response status code for this operation"""
    raw_response: httpx.Response
    r"""Raw HTTP response; suitable for custom response parsing"""
    configurations: NotRequired[List[components_configuration.ConfigurationTypedDict]]
    r"""An array of configurations"""


class GetConfigurationsResponse(BaseModel):
    content_type: str
    r"""HTTP response content type for this operation"""

    status_code: int
    r"""HTTP response status code for this operation"""

    raw_response: httpx.Response
    r"""Raw HTTP response; suitable for custom response parsing"""

    configurations: Optional[List[components_configuration.Configuration]] = None
    r"""An array of configurations"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum
from honeyhive.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Any, Dict, List, Optional, TypedDict
from typing_extensions import NotRequired


class EventType(str, Enum):
    r"""Specify whether the event is of \"session\", \"model\", \"tool\" or \"chain\" type"""

    SESSION = "session"
    MODEL = "model"
    TOOL = "tool"
    CHAIN = "chain"


class EventTypedDict(TypedDict):
    project_id: NotRequired[str]
    r"""Name of project associated with the event"""
    source: NotRequired[str]
    r"""Source of the event - production, staging, etc"""
    event_name: NotRequired[str]
    r"""Name of the event"""
    event_type: NotRequired[EventType]
    r"""Specify whether the event is of \"session\", \"model\", \"tool\" or \"chain\" type"""
    event_id: NotRequired[str]
    r"""Unique id of the event, if not set, it will be auto-generated"""
    session_id: NotRequired[str]
    r"""Unique id of the session associated with the event, if not set, it will be auto-generated"""
    parent_id: NotRequired[Nullable[str]]
    r"""Id of the parent event if nested"""
    children_ids: NotRequired[List[str]]
    r"""Id of events that are nested within the event"""
    config: NotRequired[Dict[str, Any]]
    r"""Associated configuration JSON for the event - model name, vector index name, etc"""
    inputs: NotRequired[Dict[str, Any]]
    r"""Input JSON given to the event - prompt, chunks, etc"""
    outputs: NotRequired[Dict[str, Any]]
    r"""Final output JSON of the event"""
    error: NotRequired[Nullable[str]]
    r"""Any error description if event failed"""
    start_time: NotRequired[float]
    r"""UTC timestamp (in milliseconds) for the event start"""
    end_time: NotRequired[int]
    r"""UTC timestamp (in milliseconds) for the event end"""
    duration: NotRequired[float]
    r"""How long the event took in milliseconds"""
    metadata: NotRequired[Dict[str, Any]]
    r"""Any system or application metadata associated with the event"""
    feedback: NotRequired[Dict[str, Any]]
    r"""Any user feedback provided for the event output"""
    metrics: NotRequired[Dict[str, Any]]
    r"""Any values computed over the output of the event"""
    user_properties: NotRequired[Dict[str, Any]]
    r"""Any user properties associated with the event"""


class Event(BaseModel):
    project_id: Optional[str] = None
    r"""Name of project associated with the event"""

    source: Optional[str] = None
    r"""Source of the event - production, staging, etc"""

    event_name: Optional[str] = None
    r"""Name of the event"""

    event_type: Optional[EventType] = None
    r"""Specify whether the event is of \"session\", \"model\", \"tool\" or \"chain\" type"""

    event_id: Optional[str] = None
    r"""Unique id of the event, if not set, it will be auto-generated"""

    session_id: Optional[str] = None
    r"""Unique id of the session associated with the event, if not set, it will be auto-generated"""

    parent_id: OptionalNullable[str] = UNSET
    r"""Id of the parent event if nested"""

    children_ids: Optional[List[str]] = None
    r"""Id of events that are nested within the event"""

    config: Optional[Dict[str, Any]] = None
    r"""Associated configuration JSON for the event - model name, vector index name, etc"""

    inputs: Optional[Dict[str, Any]] = None
    r"""Input JSON given to the event - prompt, chunks, etc"""

    outputs: Optional[Dict[str, Any]] = None
    r"""Final output JSON of the event"""

    error: OptionalNullable[str] = UNSET
    r"""Any error description if event failed"""

    start_time: Optional[float] = None
    r"""UTC timestamp (in milliseconds) for the event start"""

    end_time: Optional[int] = None
    r"""UTC timestamp (in milliseconds) for the event end"""

    duration: Optional[float] = None
    r"""How long the event took in milliseconds"""

    metadata: Optional[Dict[str, Any]] = None
    r"""Any system or application metadata associated with the event"""

    feedback: Optional[Dict[str, Any]] = None
    r"""Any user feedback provided for the event output"""

    metrics: Optional[Dict[str, Any]] = None
    r"""Any values computed over the output of the event"""

    user_properties: Optional[Dict[str, Any]] = None
    r"""Any user properties associated with the event"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "project_id",
            "source",
            "event_name",
            "event_type",
            "event_id",
            "session_id",
            "parent_id",
            "children_ids",
            "config",
            "inputs",
            "outputs",
            "error",
            "start_time",
            "end_time",
            "duration",
            "metadata",
            "feedback",
            "metrics",
            "user_properties",
        ]
        nullable_fields = ["parent_id", "error"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in self.model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum
from honeyhive.types import BaseModel
from typing import Any, Dict, List, Optional, TypedDict
from typing_extensions import NotRequired


class CreateEventRequestEventType(str, Enum):
    r"""Specify whether the event is of \"model\", \"tool\" or \"chain\" type"""

    MODEL = "model"
    TOOL = "tool"
    CHAIN = "chain"


class CreateEventRequestTypedDict(TypedDict):
    project: str
    r"""Project associated with the event"""
    source: str
    r"""Source of the event - production, staging, etc"""
    event_name: str
    r"""Name of the event"""
    event_type: CreateEventRequestEventType
    r"""Specify whether the event is of \"model\", \"tool\" or \"chain\" type"""
    config: Dict[str, Any]
    r"""Associated configuration JSON for the event - model name, vector index name, etc"""
    inputs: Dict[str, Any]
    r"""Input JSON given to the event - prompt, chunks, etc"""
    duration: float
    r"""How long the event took in milliseconds"""
    event_id: NotRequired[str]
    r"""Unique id of the event, if not set, it will be auto-generated"""
    session_id: NotRequired[str]
    r"""Unique id of the session associated with the event, if not set, it will be auto-generated"""
    parent_id: NotRequired[str]
    r"""Id of the parent event if nested"""
    children_ids: NotRequired[List[str]]
    r"""Id of events that are nested within the event"""
    outputs: NotRequired[Dict[str, Any]]
    r"""Final output JSON of the event"""
    error: NotRequired[str]
    r"""Any error description if event failed"""
    start_time: NotRequired[float]
    r"""UTC timestamp (in milliseconds) for the event start"""
    end_time: NotRequired[int]
    r"""UTC timestamp (in milliseconds) for the event end"""
    metadata: NotRequired[Dict[str, Any]]
    r"""Any system or application metadata associated with the event"""
    feedback: NotRequired[Dict[str, Any]]
    r"""Any user feedback provided for the event output"""
    metrics: NotRequired[Dict[str, Any]]
    r"""Any values computed over the output of the event"""
    user_properties: NotRequired[Dict[str, Any]]
    r"""Any user properties associated with the event"""


class CreateEventRequest(BaseModel):
    project: str
    r"""Project associated with the event"""

    source: str
    r"""Source of the event - production, staging, etc"""

    event_name: str
    r"""Name of the event"""

    event_type: CreateEventRequestEventType
    r"""Specify whether the event is of \"model\", \"tool\" or \"chain\" type"""

    config: Dict[str, Any]
    r"""Associated configuration JSON for the event - model name, vector index name, etc"""

    inputs: Dict[str, Any]
    r"""Input JSON given to the event - prompt, chunks, etc"""

    duration: float
    r"""How long the event took in milliseconds"""

    event_id: Optional[str] = None
    r"""Unique id of the event, if not set, it will be auto-generated"""

    session_id: Optional[str] = None
    r"""Unique id of the session associated with the event, if not set, it will be auto-generated"""

    parent_id: Optional[str] = None
    r"""Id of the parent event if nested"""

    children_ids: Optional[List[str]] = None
    r"""Id of events that are nested within the event"""

    outputs: Optional[Dict[str, Any]] = None
    r"""Final output JSON of the event"""

    error: Optional[str] = None
    r"""Any error description if event failed"""

    start_time: Optional[float] = None
    r"""UTC timestamp (in milliseconds) for the event start"""

    end_time: Optional[int] = None
    r"""UTC timestamp (in milliseconds) for the event end"""

    metadata: Optional[Dict[str, Any]] = None
    r"""Any system or application metadata associated with the event"""

    feedback: Optional[Dict[str, Any]] = None
    r"""Any user feedback provided for the event output"""

    metrics: Optional[Dict[str, Any]] = None
    r"""Any values computed over the output of the event"""

    user_properties: Optional[Dict[str, Any]] = None
    r"""Any user properties associated with the event"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum
from honeyhive.types import BaseModel
import pydantic
from typing import Any, Dict, Optional, TypedDict
from typing_extensions import Annotated, NotRequired


class ToolType(str, Enum):
    FUNCTION = "function"
    TOOL = "tool"


class ToolTypedDict(TypedDict):
    task: str
    r"""Name of the project associated with this tool"""
    name: str
    parameters: Dict[str, Any]
    r"""These can be function call params or plugin call params"""
    tool_type: ToolType
    id: NotRequired[str]
    description: NotRequired[str]


class Tool(BaseModel):
    task: str
    r"""Name of the project associated with this tool"""

    name: str

    parameters: Dict[str, Any]
    r"""These can be function call params or plugin call params"""

    tool_type: ToolType

    id: Annotated[Optional[str], pydantic.Field(alias="_id")] = None

    description: Optional[str] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from honeyhive.models.components import event as components_event
from honeyhive.types import BaseModel
from honeyhive.utils import FieldMetadata, PathParamMetadata
import httpx
from typing import Optional, TypedDict
from typing_extensions import Annotated, NotRequired


class GetSessionRequestTypedDict(TypedDict):
    session_id: str


class GetSessionRequest(BaseModel):
    session_id: Annotated[
        str, FieldMetadata(path=PathParamMetadata(style="simple", explode=False))
    ]


class GetSessionResponseTypedDict(TypedDict):
    content_type: str
    r"""HTTP response content type for this operation"""
    status_code: int
    r"""HTTP response status code for this operation"""
    raw_response: httpx.Response
    r"""Raw HTTP response; suitable for custom response parsing"""
    event: NotRequired[components_event.EventTypedDict]
    r"""Session details"""


class GetSessionResponse(BaseModel):
    content_type: str
    r"""HTTP response content type for this operation"""

    status_code: int
    r"""HTTP response status code for this operation"""

    raw_response: httpx.Response
    r"""Raw HTTP response; suitable for custom response parsing"""

    event: Optional[components_event.Event] = None
    r"""Session details"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .evaluationrun import EvaluationRun, EvaluationRunTypedDict
from honeyhive.types import BaseModel
from typing import List, Optional, TypedDict
from typing_extensions import NotRequired


class GetRunsResponseTypedDict(TypedDict):
    evaluations: NotRequired[List[EvaluationRunTypedDict]]


class GetRunsResponse(BaseModel):
    evaluations: Optional[List[EvaluationRun]] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from honeyhive.types import BaseModel
import httpx
from typing import TypedDict


class UpdateDatasetResponseTypedDict(TypedDict):
    content_type: str
    r"""HTTP response content type for this operation"""
    status_code: int
    r"""HTTP response status code for this operation"""
    raw_response: httpx.Response
    r"""Raw HTTP response; suitable for custom response parsing"""


class UpdateDatasetResponse(BaseModel):
    content_type: str
    r"""HTTP response content type for this operation"""

    status_code: int
    r"""HTTP response status code for this operation"""

    raw_response: httpx.Response
    r"""Raw HTTP response; suitable for custom response parsing"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from honeyhive.types import BaseModel
import httpx
import pydantic
from typing import Optional, TypedDict
from typing_extensions import Annotated, NotRequired


class CreateDatasetResultTypedDict(TypedDict):
    inserted_id: NotRequired[str]
    r"""UUID for the created dataset"""


class CreateDatasetResult(BaseModel):
    inserted_id: Annotated[Optional[str], pydantic.Field(alias="insertedId")] = None
    r"""UUID for the created dataset"""


class CreateDatasetResponseBodyTypedDict(TypedDict):
    r"""Successful creation"""

    inserted: NotRequired[bool]
    result: NotRequired[CreateDatasetResultTypedDict]


class CreateDatasetResponseBody(BaseModel):
    r"""Successful creation"""

    inserted: Optional[bool] = None

    result: Optional[CreateDatasetResult] = None


class CreateDatasetResponseTypedDict(TypedDict):
    content_type: str
    r"""HTTP response content type for this operation"""
    status_code: int
    r"""HTTP response status code for this operation"""
    raw_response: httpx.Response
    r"""Raw HTTP response; suitable for custom response parsing"""
    object: NotRequired[CreateDatasetResponseBodyTypedDict]
    r"""Successful creation"""


class CreateDatasetResponse(BaseModel):
    content_type: str
    r"""HTTP response content type for this operation"""

    status_code: int
    r"""HTTP response status code for this operation"""

    raw_response: httpx.Response
    r"""Raw HTTP response; suitable for custom response parsing"""

    object: Optional[CreateDatasetResponseBody] = None
    r"""Successful creation"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum
from honeyhive.types import BaseModel
from typing import Any, Dict, List, Optional, TypedDict
from typing_extensions import NotRequired


class Status(str, Enum):
    r"""The status of the run"""

    PENDING = "pending"
    COMPLETED = "completed"


class CreateRunRequestTypedDict(TypedDict):
    project: str
    r"""The UUID of the project this run is associated with"""
    name: str
    r"""The name of the run to be displayed"""
    event_ids: List[str]
    r"""The UUIDs of the sessions/events this run is associated with"""
    dataset_id: NotRequired[str]
    r"""The UUID of the dataset this run is associated with"""
    datapoint_ids: NotRequired[List[str]]
    r"""The UUIDs of the datapoints from the original dataset this run is associated with"""
    configuration: NotRequired[Dict[str, Any]]
    r"""The configuration being used for this run"""
    metadata: NotRequired[Dict[str, Any]]
    r"""Additional metadata for the run"""
    status: NotRequired[Status]
    r"""The status of the run"""


class CreateRunRequest(BaseModel):
    project: str
    r"""The UUID of the project this run is associated with"""

    name: str
    r"""The name of the run to be displayed"""

    event_ids: List[str]
    r"""The UUIDs of the sessions/events this run is associated with"""

    dataset_id: Optional[str] = None
    r"""The UUID of the dataset this run is associated with"""

    datapoint_ids: Optional[List[str]] = None
    r"""The UUIDs of the datapoints from the original dataset this run is associated with"""

    configuration: Optional[Dict[str, Any]] = None
    r"""The configuration being used for this run"""

    metadata: Optional[Dict[str, Any]] = None
    r"""Additional metadata for the run"""

    status: Optional[Status] = None
    r"""The status of the run"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from honeyhive.types import BaseModel
from honeyhive.utils import FieldMetadata, QueryParamMetadata
import httpx
from typing import TypedDict
from typing_extensions import Annotated


class DeleteDatasetRequestTypedDict(TypedDict):
    dataset_id: str
    r"""The unique identifier of the dataset to be deleted like `663876ec4611c47f4970f0c3`"""


class DeleteDatasetRequest(BaseModel):
    dataset_id: Annotated[
        str, FieldMetadata(query=QueryParamMetadata(style="form", explode=True))
    ]
    r"""The unique identifier of the dataset to be deleted like `663876ec4611c47f4970f0c3`"""


class DeleteDatasetResponseTypedDict(TypedDict):
    content_type: str
    r"""HTTP response content type for this operation"""
    status_code: int
    r"""HTTP response status code for this operation"""
    raw_response: httpx.Response
    r"""Raw HTTP response; suitable for custom response parsing"""


class DeleteDatasetResponse(BaseModel):
    content_type: str
    r"""HTTP response content type for this operation"""

    status_code: int
    r"""HTTP response status code for this operation"""

    raw_response: httpx.Response
    r"""Raw HTTP response; suitable for custom response parsing"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from honeyhive.types import BaseModel
from honeyhive.utils import FieldMetadata, PathParamMetadata, RequestMetadata
import httpx
from typing import Any, Dict, List, Optional, TypedDict
from typing_extensions import Annotated, NotRequired


class MappingTypedDict(TypedDict):
    r"""Mapping of keys in the data object to be used as inputs, ground truth, and history, everything else goes into metadata"""

    inputs: List[str]
    r"""List of keys in the data object to be used as inputs"""
    ground_truth: List[str]
    r"""List of keys in the data object to be used as ground truth"""
    history: List[str]
    r"""List of keys in the data object to be used as chat history, can be empty list if not needed"""


class Mapping(BaseModel):
    r"""Mapping of keys in the data object to be used as inputs, ground truth, and history, everything else goes into metadata"""

    inputs: List[str]
    r"""List of keys in the data object to be used as inputs"""

    ground_truth: List[str]
    r"""List of keys in the data object to be used as ground truth"""

    history: List[str]
    r"""List of keys in the data object to be used as chat history, can be empty list if not needed"""


class AddDatapointsRequestBodyTypedDict(TypedDict):
    project: str
    r"""Name of the project associated with this dataset like `New Project`"""
    data: List[Dict[str, Any]]
    r"""List of JSON objects to be added as datapoints"""
    mapping: MappingTypedDict
    r"""Mapping of keys in the data object to be used as inputs, ground truth, and history, everything else goes into metadata"""


class AddDatapointsRequestBody(BaseModel):
    project: str
    r"""Name of the project associated with this dataset like `New Project`"""

    data: List[Dict[str, Any]]
    r"""List of JSON objects to be added as datapoints"""

    mapping: Mapping
    r"""Mapping of keys in the data object to be used as inputs, ground truth, and history, everything else goes into metadata"""


class AddDatapointsRequestTypedDict(TypedDict):
    dataset_id: str
    r"""The unique identifier of the dataset to add datapoints to like  `663876ec4611c47f4970f0c3`"""
    request_body: AddDatapointsRequestBodyTypedDict


class AddDatapointsRequest(BaseModel):
    dataset_id: Annotated[
        str, FieldMetadata(path=PathParamMetadata(style="simple", explode=False))
    ]
    r"""The unique identifier of the dataset to add datapoints to like  `663876ec4611c47f4970f0c3`"""

    request_body: Annotated[
        AddDatapointsRequestBody,
        FieldMetadata(request=RequestMetadata(media_type="application/json")),
    ]


class AddDatapointsResponseBodyTypedDict(TypedDict):
    r"""Successful addition"""

    inserted: NotRequired[bool]
    datapoint_ids: NotRequired[List[str]]
    r"""List of unique datapoint ids added to the dataset"""


class AddDatapointsResponseBody(BaseModel):
    r"""Successful addition"""

    inserted: Optional[bool] = None

    datapoint_ids: Optional[List[str]] = None
    r"""List of unique datapoint ids added to the dataset"""


class AddDatapointsResponseTypedDict(TypedDict):
    content_type: str
    r"""HTTP response content type for this operation"""
    status_code: int
    r"""HTTP response status code for this operation"""
    raw_response: httpx.Response
    r"""Raw HTTP response; suitable for custom response parsing"""
    object: NotRequired[AddDatapointsResponseBodyTypedDict]
    r"""Successful addition"""


class AddDatapointsResponse(BaseModel):
    content_type: str
    r"""HTTP response content type for this operation"""

    status_code: int
    r"""HTTP response status code for this operation"""

    raw_response: httpx.Response
    r"""Raw HTTP response; suitable for custom response parsing"""

    object: Optional[AddDatapointsResponseBody] = None
    r"""Successful addition"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from honeyhive import utils
from honeyhive.types import BaseModel
import httpx
import pydantic
from typing import List, Optional
from typing_extensions import Annotated


class CreateEventBatchResponseBodyData(BaseModel):
    event_ids: Optional[List[str]] = None

    errors: Optional[List[str]] = None

    success: Optional[bool] = None

    raw_response: Annotated[Optional[httpx.Response], pydantic.Field(exclude=True)] = (
        None
    )
    r"""Raw HTTP response; suitable for custom response parsing"""


class CreateEventBatchResponseBody(Exception):
    r"""Events partially created"""

    data: CreateEventBatchResponseBodyData

    def __init__(self, data: CreateEventBatchResponseBodyData):
        self.data = data

    def __str__(self) -> str:
        return utils.marshal_json(self.data, CreateEventBatchResponseBodyData)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .basesdk import BaseSDK
from honeyhive import utils
from honeyhive._hooks import HookContext
from honeyhive.models import errors, operations
from honeyhive.types import BaseModel, OptionalNullable, UNSET
from typing import Any, Optional, Union, cast


class Events(BaseSDK):
    def create_event(
        self,
        *,
        request: Union[
            operations.CreateEventRequestBody,
            operations.CreateEventRequestBodyTypedDict,
        ],
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
    ) -> operations.CreateEventResponse:
        r"""Create a new event

        Please refer to our instrumentation guide for detailed information

        :param request: The request object to send.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url

        if not isinstance(request, BaseModel):
            request = utils.unmarshal(request, operations.CreateEventRequestBody)
        request = cast(operations.CreateEventRequestBody, request)

        req = self.build_request(
            method="POST",
            path="/events",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request, False, False, "json", operations.CreateEventRequestBody
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                operation_id="createEvent",
                oauth2_scopes=[],
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["4XX", "5XX"],
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "200", "application/json"):
            return operations.CreateEventResponse(
                object=utils.unmarshal_json(
                    http_res.text, Optional[operations.CreateEventResponseBody]
                ),
                status_code=http_res.status_code,
                content_type=http_res.headers.get("Content-Type") or "",
                raw_response=http_res,
            )
        if utils.match_response(http_res, ["4XX", "5XX"], "*"):
            raise errors.SDKError(
                "API error occurred", http_res.status_code, http_res.text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        raise errors.SDKError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res.text,
            http_res,
        )

    async def create_event_async(
        self,
        *,
        request: Union[
            operations.CreateEventRequestBody,
            operations.CreateEventRequestBodyTypedDict,
        ],
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
    ) -> operations.CreateEventResponse:
        r"""Create a new event

        Please refer to our instrumentation guide for detailed information

        :param request: The request object to send.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url

        if not isinstance(request, BaseModel):
            request = utils.unmarshal(request, operations.CreateEventRequestBody)
        request = cast(operations.CreateEventRequestBody, request)

        req = self.build_request_async(
            method="POST",
            path="/events",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request, False, False, "json", operations.CreateEventRequestBody
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                operation_id="createEvent",
                oauth2_scopes=[],
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["4XX", "5XX"],
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "200", "application/json"):
            return operations.CreateEventResponse(
                object=utils.unmarshal_json(
                    http_res.text, Optional[operations.CreateEventResponseBody]
                ),
                status_code=http_res.status_code,
                content_type=http_res.headers.get("Content-Type") or "",
                raw_response=http_res,
            )
        if utils.match_response(http_res, ["4XX", "5XX"], "*"):
            raise errors.SDKError(
                "API error occurred", http_res.status_code, http_res.text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        raise errors.SDKError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res.text,
            http_res,
        )

    def update_event(
        self,
        *,
        request: Union[
            operations.UpdateEventRequestBody,
            operations.UpdateEventRequestBodyTypedDict,
        ],
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
    ) -> operations.UpdateEventResponse:
        r"""Update an event

        :param request: The request object to send.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url

        if not isinstance(request, BaseModel):
            request = utils.unmarshal(request, operations.UpdateEventRequestBody)
        request = cast(operations.UpdateEventRequestBody, request)

        req = self.build_request(
            method="PUT",
            path="/events",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="*/*",
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request, False, False, "json", operations.UpdateEventRequestBody
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                operation_id="updateEvent",
                oauth2_scopes=[],
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "4XX", "5XX"],
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "200", "*"):
            return operations.UpdateEventResponse(
                status_code=http_res.status_code,
                content_type=http_res.headers.get("Content-Type") or "",
                raw_response=http_res,
            )
        if utils.match_response(http_res, ["400", "4XX", "5XX"], "*"):
            raise errors.SDKError(
                "API error occurred", http_res.status_code, http_res.text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        raise errors.SDKError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res.text,
            http_res,
        )

    async def update_event_async(
        self,
        *,
        request: Union[
            operations.UpdateEventRequestBody,
            operations.UpdateEventRequestBodyTypedDict,
        ],
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
    ) -> operations.UpdateEventResponse:
        r"""Update an event

        :param request: The request object to send.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url

        if not isinstance(request, BaseModel):
            request = utils.unmarshal(request, operations.UpdateEventRequestBody)
        request = cast(operations.UpdateEventRequestBody, request)

        req = self.build_request_async(
            method="PUT",
            path="/events",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="*/*",
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request, False, False, "json", operations.UpdateEventRequestBody
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                operation_id="updateEvent",
                oauth2_scopes=[],
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "4XX", "5XX"],
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "200", "*"):
            return operations.UpdateEventResponse(
                status_code=http_res.status_code,
                content_type=http_res.headers.get("Content-Type") or "",
                raw_response=http_res,
            )
        if utils.match_response(http_res, ["400", "4XX", "5XX"], "*"):
            raise errors.SDKError(
                "API error occurred", http_res.status_code, http_res.text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        raise errors.SDKError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res.text,
            http_res,
        )

    def get_events(
        self,
        *,
        request: Union[
            operations.GetEventsRequestBody, operations.GetEventsRequestBodyTypedDict
        ],
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
    ) -> operations.GetEventsResponse:
        r"""Retrieve events based on filters

        :param request: The request object to send.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url

        if not isinstance(request, BaseModel):
            request = utils.unmarshal(request, operations.GetEventsRequestBody)
        request = cast(operations.GetEventsRequestBody, request)

        req = self.build_request(
            method="POST",
            path="/events/export",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request, False, False, "json", operations.GetEventsRequestBody
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                operation_id="getEvents",
                oauth2_scopes=[],
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["4XX", "5XX"],
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "200", "application/json"):
            return operations.GetEventsResponse(
                object=utils.unmarshal_json(
                    http_res.text, Optional[operations.GetEventsResponseBody]
                ),
                status_code=http_res.status_code,
                content_type=http_res.headers.get("Content-Type") or "",
                raw_response=http_res,
            )
        if utils.match_response(http_res, ["4XX", "5XX"], "*"):
            raise errors.SDKError(
                "API error occurred", http_res.status_code, http_res.text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        raise errors.SDKError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res.text,
            http_res,
        )

    async def get_events_async(
        self,
        *,
        request: Union[
            operations.GetEventsRequestBody, operations.GetEventsRequestBodyTypedDict
        ],
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
    ) -> operations.GetEventsResponse:
        r"""Retrieve events based on filters

        :param request: The request object to send.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url

        if not isinstance(request, BaseModel):
            request = utils.unmarshal(request, operations.GetEventsRequestBody)
        request = cast(operations.GetEventsRequestBody, request)

        req = self.build_request_async(
            method="POST",
            path="/events/export",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request, False, False, "json", operations.GetEventsRequestBody
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                operation_id="getEvents",
                oauth2_scopes=[],
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["4XX", "5XX"],
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "200", "application/json"):
            return operations.GetEventsResponse(
                object=utils.unmarshal_json(
                    http_res.text, Optional[operations.GetEventsResponseBody]
                ),
                status_code=http_res.status_code,
                content_type=http_res.headers.get("Content-Type") or "",
                raw_response=http_res,
            )
        if utils.match_response(http_res, ["4XX", "5XX"], "*"):
            raise errors.SDKError(
                "API error occurred", http_res.status_code, http_res.text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        raise errors.SDKError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res.text,
            http_res,
        )

    def create_model_event(
        self,
        *,
        request: Union[
            operations.CreateModelEventRequestBody,
            operations.CreateModelEventRequestBodyTypedDict,
        ],
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
    ) -> operations.CreateModelEventResponse:
        r"""Create a new model event

        Please refer to our instrumentation guide for detailed information

        :param request: The request object to send.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url

        if not isinstance(request, BaseModel):
            request = utils.unmarshal(request, operations.CreateModelEventRequestBody)
        request = cast(operations.CreateModelEventRequestBody, request)

        req = self.build_request(
            method="POST",
            path="/events/model",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request, False, False, "json", operations.CreateModelEventRequestBody
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                operation_id="createModelEvent",
                oauth2_scopes=[],
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["4XX", "5XX"],
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "200", "application/json"):
            return operations.CreateModelEventResponse(
                object=utils.unmarshal_json(
                    http_res.text, Optional[operations.CreateModelEventResponseBody]
                ),
                status_code=http_res.status_code,
                content_type=http_res.headers.get("Content-Type") or "",
                raw_response=http_res,
            )
        if utils.match_response(http_res, ["4XX", "5XX"], "*"):
            raise errors.SDKError(
                "API error occurred", http_res.status_code, http_res.text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        raise errors.SDKError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res.text,
            http_res,
        )

    async def create_model_event_async(
        self,
        *,
        request: Union[
            operations.CreateModelEventRequestBody,
            operations.CreateModelEventRequestBodyTypedDict,
        ],
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
    ) -> operations.CreateModelEventResponse:
        r"""Create a new model event

        Please refer to our instrumentation guide for detailed information

        :param request: The request object to send.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url

        if not isinstance(request, BaseModel):
            request = utils.unmarshal(request, operations.CreateModelEventRequestBody)
        request = cast(operations.CreateModelEventRequestBody, request)

        req = self.build_request_async(
            method="POST",
            path="/events/model",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request, False, False, "json", operations.CreateModelEventRequestBody
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                operation_id="createModelEvent",
                oauth2_scopes=[],
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["4XX", "5XX"],
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "200", "application/json"):
            return operations.CreateModelEventResponse(
                object=utils.unmarshal_json(
                    http_res.text, Optional[operations.CreateModelEventResponseBody]
                ),
                status_code=http_res.status_code,
                content_type=http_res.headers.get("Content-Type") or "",
                raw_response=http_res,
            )
        if utils.match_response(http_res, ["4XX", "5XX"], "*"):
            raise errors.SDKError(
                "API error occurred", http_res.status_code, http_res.text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        raise errors.SDKError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res.text,
            http_res,
        )

    def create_event_batch(
        self,
        *,
        request: Union[
            operations.CreateEventBatchRequestBody,
            operations.CreateEventBatchRequestBodyTypedDict,
        ],
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
    ) -> operations.CreateEventBatchResponse:
        r"""Create a batch of events

        Please refer to our instrumentation guide for detailed information

        :param request: The request object to send.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url

        if not isinstance(request, BaseModel):
            request = utils.unmarshal(request, operations.CreateEventBatchRequestBody)
        request = cast(operations.CreateEventBatchRequestBody, request)

        req = self.build_request(
            method="POST",
            path="/events/batch",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request, False, False, "json", operations.CreateEventBatchRequestBody
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                operation_id="createEventBatch",
                oauth2_scopes=[],
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return operations.CreateEventBatchResponse(
                object=utils.unmarshal_json(
                    http_res.text, Optional[operations.CreateEventBatchResponseBody]
                ),
                status_code=http_res.status_code,
                content_type=http_res.headers.get("Content-Type") or "",
                raw_response=http_res,
            )
        if utils.match_response(http_res, "500", "application/json"):
            data = utils.unmarshal_json(
                http_res.text, errors.CreateEventBatchResponseBodyData
            )
            data.raw_response = http_res
            raise errors.CreateEventBatchResponseBody(data=data)
        if utils.match_response(http_res, ["4XX", "5XX"], "*"):
            raise errors.SDKError(
                "API error occurred", http_res.status_code, http_res.text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        raise errors.SDKError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res.text,
            http_res,
        )

    async def create_event_batch_async(
        self,
        *,
        request: Union[
            operations.CreateEventBatchRequestBody,
            operations.CreateEventBatchRequestBodyTypedDict,
        ],
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
    ) -> operations.CreateEventBatchResponse:
        r"""Create a batch of events

        Please refer to our instrumentation guide for detailed information

        :param request: The request object to send.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url

        if not isinstance(request, BaseModel):
            request = utils.unmarshal(request, operations.CreateEventBatchRequestBody)
        request = cast(operations.CreateEventBatchRequestBody, request)

        req = self.build_request_async(
            method="POST",
            path="/events/batch",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request, False, False, "json", operations.CreateEventBatchRequestBody
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                operation_id="createEventBatch",
                oauth2_scopes=[],
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return operations.CreateEventBatchResponse(
                object=utils.unmarshal_json(
                    http_res.text, Optional[operations.CreateEventBatchResponseBody]
                ),
                status_code=http_res.status_code,
                content_type=http_res.headers.get("Content-Type") or "",
                raw_response=http_res,
            )
        if utils.match_response(http_res, "500", "application/json"):
            data = utils.unmarshal_json(
                http_res.text, errors.CreateEventBatchResponseBodyData
            )
            data.raw_response = http_res
            raise errors.CreateEventBatchResponseBody(data=data)
        if utils.match_response(http_res, ["4XX", "5XX"], "*"):
            raise errors.SDKError(
                "API error occurred", http_res.status_code, http_res.text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        raise errors.SDKError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res.text,
            http_res,
        )

    def create_model_event_batch(
        self,
        *,
        request: Union[
            operations.CreateModelEventBatchRequestBody,
            operations.CreateModelEventBatchRequestBodyTypedDict,
        ],
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
    ) -> operations.CreateModelEventBatchResponse:
        r"""Create a batch of model events

        Please refer to our instrumentation guide for detailed information

        :param request: The request object to send.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url

        if not isinstance(request, BaseModel):
            request = utils.unmarshal(
                request, operations.CreateModelEventBatchRequestBody
            )
        request = cast(operations.CreateModelEventBatchRequestBody, request)

        req = self.build_request(
            method="POST",
            path="/events/model/batch",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request,
                False,
                False,
                "json",
                operations.CreateModelEventBatchRequestBody,
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                operation_id="createModelEventBatch",
                oauth2_scopes=[],
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return operations.CreateModelEventBatchResponse(
                object=utils.unmarshal_json(
                    http_res.text,
                    Optional[operations.CreateModelEventBatchResponseBody],
                ),
                status_code=http_res.status_code,
                content_type=http_res.headers.get("Content-Type") or "",
                raw_response=http_res,
            )
        if utils.match_response(http_res, "500", "application/json"):
            data = utils.unmarshal_json(
                http_res.text, errors.CreateModelEventBatchResponseBodyData
            )
            data.raw_response = http_res
            raise errors.CreateModelEventBatchResponseBody(data=data)
        if utils.match_response(http_res, ["4XX", "5XX"], "*"):
            raise errors.SDKError(
                "API error occurred", http_res.status_code, http_res.text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        raise errors.SDKError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res.text,
            http_res,
        )

    async def create_model_event_batch_async(
        self,
        *,
        request: Union[
            operations.CreateModelEventBatchRequestBody,
            operations.CreateModelEventBatchRequestBodyTypedDict,
        ],
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
    ) -> operations.CreateModelEventBatchResponse:
        r"""Create a batch of model events

        Please refer to our instrumentation guide for detailed information

        :param request: The request object to send.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url

        if not isinstance(request, BaseModel):
            request = utils.unmarshal(
                request, operations.CreateModelEventBatchRequestBody
            )
        request = cast(operations.CreateModelEventBatchRequestBody, request)

        req = self.build_request_async(
            method="POST",
            path="/events/model/batch",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request,
                False,
                False,
                "json",
                operations.CreateModelEventBatchRequestBody,
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                operation_id="createModelEventBatch",
                oauth2_scopes=[],
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return operations.CreateModelEventBatchResponse(
                object=utils.unmarshal_json(
                    http_res.text,
                    Optional[operations.CreateModelEventBatchResponseBody],
                ),
                status_code=http_res.status_code,
                content_type=http_res.headers.get("Content-Type") or "",
                raw_response=http_res,
            )
        if utils.match_response(http_res, "500", "application/json"):
            data = utils.unmarshal_json(
                http_res.text, errors.CreateModelEventBatchResponseBodyData
            )
            data.raw_response = http_res
            raise errors.CreateModelEventBatchResponseBody(data=data)
        if utils.match_response(http_res, ["4XX", "5XX"], "*"):
            raise errors.SDKError(
                "API error occurred", http_res.status_code, http_res.text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        raise errors.SDKError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res.text,
            http_res,
        )



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from honeyhive.types import BaseModel
from typing import List, Optional, TypedDict, Union
from typing_extensions import NotRequired


ValuesTypedDict = Union[float, bool]


Values = Union[float, bool]


class ExperimentResultResponseDatapointsTypedDict(TypedDict):
    passed: NotRequired[List[str]]
    failed: NotRequired[List[str]]


class ExperimentResultResponseDatapoints(BaseModel):
    passed: Optional[List[str]] = None

    failed: Optional[List[str]] = None


class DetailsTypedDict(TypedDict):
    metric_name: NotRequired[str]
    metric_type: NotRequired[str]
    event_name: NotRequired[str]
    event_type: NotRequired[str]
    aggregate: NotRequired[float]
    values: NotRequired[List[ValuesTypedDict]]
    datapoints: NotRequired[ExperimentResultResponseDatapointsTypedDict]


class Details(BaseModel):
    metric_name: Optional[str] = None

    metric_type: Optional[str] = None

    event_name: Optional[str] = None

    event_type: Optional[str] = None

    aggregate: Optional[float] = None

    values: Optional[List[Values]] = None

    datapoints: Optional[ExperimentResultResponseDatapoints] = None


class MetricsTypedDict(TypedDict):
    aggregation_function: NotRequired[str]
    details: NotRequired[List[DetailsTypedDict]]


class Metrics(BaseModel):
    aggregation_function: Optional[str] = None

    details: Optional[List[Details]] = None


ValueTypedDict = Union[float, bool]


Value = Union[float, bool]


class ExperimentResultResponseMetricsTypedDict(TypedDict):
    name: NotRequired[str]
    event_name: NotRequired[str]
    event_type: NotRequired[str]
    value: NotRequired[ValueTypedDict]
    passed: NotRequired[bool]


class ExperimentResultResponseMetrics(BaseModel):
    name: Optional[str] = None

    event_name: Optional[str] = None

    event_type: Optional[str] = None

    value: Optional[Value] = None

    passed: Optional[bool] = None


class DatapointsTypedDict(TypedDict):
    datapoint_id: NotRequired[str]
    session_id: NotRequired[str]
    passed: NotRequired[bool]
    metrics: NotRequired[List[ExperimentResultResponseMetricsTypedDict]]


class Datapoints(BaseModel):
    datapoint_id: Optional[str] = None

    session_id: Optional[str] = None

    passed: Optional[bool] = None

    metrics: Optional[List[ExperimentResultResponseMetrics]] = None


class ExperimentResultResponseTypedDict(TypedDict):
    status: NotRequired[str]
    success: NotRequired[bool]
    passed: NotRequired[List[str]]
    failed: NotRequired[List[str]]
    metrics: NotRequired[MetricsTypedDict]
    datapoints: NotRequired[List[DatapointsTypedDict]]


class ExperimentResultResponse(BaseModel):
    status: Optional[str] = None

    success: Optional[bool] = None

    passed: Optional[List[str]] = None

    failed: Optional[List[str]] = None

    metrics: Optional[Metrics] = None

    datapoints: Optional[List[Datapoints]] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from honeyhive.models.components import datapoint as components_datapoint
from honeyhive.types import BaseModel
from honeyhive.utils import FieldMetadata, QueryParamMetadata
import httpx
from typing import List, Optional, TypedDict
from typing_extensions import Annotated, NotRequired


class GetDatapointsRequestTypedDict(TypedDict):
    project: str
    r"""Project name to filter datapoints"""
    datapoint_ids: NotRequired[List[str]]
    r"""List of datapoint ids to fetch"""
    dataset_name: NotRequired[str]
    r"""Name of the dataset to get datapoints from"""


class GetDatapointsRequest(BaseModel):
    project: Annotated[
        str, FieldMetadata(query=QueryParamMetadata(style="form", explode=True))
    ]
    r"""Project name to filter datapoints"""

    datapoint_ids: Annotated[
        Optional[List[str]],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None
    r"""List of datapoint ids to fetch"""

    dataset_name: Annotated[
        Optional[str],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None
    r"""Name of the dataset to get datapoints from"""


class GetDatapointsResponseBodyTypedDict(TypedDict):
    r"""Successful response"""

    datapoints: NotRequired[List[components_datapoint.DatapointTypedDict]]


class GetDatapointsResponseBody(BaseModel):
    r"""Successful response"""

    datapoints: Optional[List[components_datapoint.Datapoint]] = None


class GetDatapointsResponseTypedDict(TypedDict):
    content_type: str
    r"""HTTP response content type for this operation"""
    status_code: int
    r"""HTTP response status code for this operation"""
    raw_response: httpx.Response
    r"""Raw HTTP response; suitable for custom response parsing"""
    object: NotRequired[GetDatapointsResponseBodyTypedDict]
    r"""Successful response"""


class GetDatapointsResponse(BaseModel):
    content_type: str
    r"""HTTP response content type for this operation"""

    status_code: int
    r"""HTTP response status code for this operation"""

    raw_response: httpx.Response
    r"""Raw HTTP response; suitable for custom response parsing"""

    object: Optional[GetDatapointsResponseBody] = None
    r"""Successful response"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from honeyhive.models.components import (
    event as components_event,
    eventfilter as components_eventfilter,
)
from honeyhive.types import BaseModel
import httpx
import pydantic
from typing import List, Optional, TypedDict
from typing_extensions import Annotated, NotRequired


class DateRangeTypedDict(TypedDict):
    dollar_gte: NotRequired[str]
    r"""ISO String for start of date time filter like `2024-04-01T22:38:19.000Z`"""
    dollar_lte: NotRequired[str]
    r"""ISO String for end of date time filter like `2024-04-01T22:38:19.000Z`"""


class DateRange(BaseModel):
    dollar_gte: Annotated[Optional[str], pydantic.Field(alias="$gte")] = None
    r"""ISO String for start of date time filter like `2024-04-01T22:38:19.000Z`"""

    dollar_lte: Annotated[Optional[str], pydantic.Field(alias="$lte")] = None
    r"""ISO String for end of date time filter like `2024-04-01T22:38:19.000Z`"""


class GetEventsRequestBodyTypedDict(TypedDict):
    project: str
    r"""Name of the project associated with the event like `New Project`"""
    filters: List[components_eventfilter.EventFilterTypedDict]
    date_range: NotRequired[DateRangeTypedDict]
    projections: NotRequired[List[str]]
    r"""Fields to include in the response"""
    limit: NotRequired[float]
    r"""Limit number of results to speed up query (default is 1000, max is 7500)"""
    page: NotRequired[float]
    r"""Page number of results (default is 1)"""


class GetEventsRequestBody(BaseModel):
    project: str
    r"""Name of the project associated with the event like `New Project`"""

    filters: List[components_eventfilter.EventFilter]

    date_range: Annotated[Optional[DateRange], pydantic.Field(alias="dateRange")] = None

    projections: Optional[List[str]] = None
    r"""Fields to include in the response"""

    limit: Optional[float] = None
    r"""Limit number of results to speed up query (default is 1000, max is 7500)"""

    page: Optional[float] = None
    r"""Page number of results (default is 1)"""


class GetEventsResponseBodyTypedDict(TypedDict):
    r"""Success"""

    events: NotRequired[List[components_event.EventTypedDict]]
    total_events: NotRequired[float]
    r"""Total number of events in the specified filter"""


class GetEventsResponseBody(BaseModel):
    r"""Success"""

    events: Optional[List[components_event.Event]] = None

    total_events: Annotated[Optional[float], pydantic.Field(alias="totalEvents")] = None
    r"""Total number of events in the specified filter"""


class GetEventsResponseTypedDict(TypedDict):
    content_type: str
    r"""HTTP response content type for this operation"""
    status_code: int
    r"""HTTP response status code for this operation"""
    raw_response: httpx.Response
    r"""Raw HTTP response; suitable for custom response parsing"""
    object: NotRequired[GetEventsResponseBodyTypedDict]
    r"""Success"""


class GetEventsResponse(BaseModel):
    content_type: str
    r"""HTTP response content type for this operation"""

    status_code: int
    r"""HTTP response status code for this operation"""

    raw_response: httpx.Response
    r"""Raw HTTP response; suitable for custom response parsing"""

    object: Optional[GetEventsResponseBody] = None
    r"""Success"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from honeyhive.types import BaseModel
from honeyhive.utils import FieldMetadata, PathParamMetadata
import httpx
from typing import TypedDict
from typing_extensions import Annotated


class DeleteConfigurationRequestTypedDict(TypedDict):
    id: str
    r"""Configuration ID like `6638187d505c6812e4043f24`"""


class DeleteConfigurationRequest(BaseModel):
    id: Annotated[
        str, FieldMetadata(path=PathParamMetadata(style="simple", explode=False))
    ]
    r"""Configuration ID like `6638187d505c6812e4043f24`"""


class DeleteConfigurationResponseTypedDict(TypedDict):
    content_type: str
    r"""HTTP response content type for this operation"""
    status_code: int
    r"""HTTP response status code for this operation"""
    raw_response: httpx.Response
    r"""Raw HTTP response; suitable for custom response parsing"""


class DeleteConfigurationResponse(BaseModel):
    content_type: str
    r"""HTTP response content type for this operation"""

    status_code: int
    r"""HTTP response status code for this operation"""

    raw_response: httpx.Response
    r"""Raw HTTP response; suitable for custom response parsing"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .basesdk import BaseSDK
from honeyhive import utils
from honeyhive._hooks import HookContext
from honeyhive.models import components, errors, operations
from honeyhive.types import BaseModel, OptionalNullable, UNSET
from typing import Optional, Union, cast


class Datasets(BaseSDK):
    def get_datasets(
        self,
        *,
        project: str,
        type_: Optional[operations.Type] = None,
        dataset_id: Optional[str] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
    ) -> operations.GetDatasetsResponse:
        r"""Get datasets

        :param project: Project Name associated with the datasets like `New Project`
        :param type: Type of the dataset - \"evaluation\" or \"fine-tuning\"
        :param dataset_id: Unique dataset ID for filtering specific dataset like `663876ec4611c47f4970f0c3`
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url

        request = operations.GetDatasetsRequest(
            project=project,
            type=type_,
            dataset_id=dataset_id,
        )

        req = self.build_request(
            method="GET",
            path="/datasets",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                operation_id="getDatasets",
                oauth2_scopes=[],
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["4XX", "5XX"],
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "200", "application/json"):
            return operations.GetDatasetsResponse(
                object=utils.unmarshal_json(
                    http_res.text, Optional[operations.GetDatasetsResponseBody]
                ),
                status_code=http_res.status_code,
                content_type=http_res.headers.get("Content-Type") or "",
                raw_response=http_res,
            )
        if utils.match_response(http_res, ["4XX", "5XX"], "*"):
            raise errors.SDKError(
                "API error occurred", http_res.status_code, http_res.text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        raise errors.SDKError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res.text,
            http_res,
        )

    async def get_datasets_async(
        self,
        *,
        project: str,
        type_: Optional[operations.Type] = None,
        dataset_id: Optional[str] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
    ) -> operations.GetDatasetsResponse:
        r"""Get datasets

        :param project: Project Name associated with the datasets like `New Project`
        :param type: Type of the dataset - \"evaluation\" or \"fine-tuning\"
        :param dataset_id: Unique dataset ID for filtering specific dataset like `663876ec4611c47f4970f0c3`
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url

        request = operations.GetDatasetsRequest(
            project=project,
            type=type_,
            dataset_id=dataset_id,
        )

        req = self.build_request_async(
            method="GET",
            path="/datasets",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                operation_id="getDatasets",
                oauth2_scopes=[],
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["4XX", "5XX"],
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "200", "application/json"):
            return operations.GetDatasetsResponse(
                object=utils.unmarshal_json(
                    http_res.text, Optional[operations.GetDatasetsResponseBody]
                ),
                status_code=http_res.status_code,
                content_type=http_res.headers.get("Content-Type") or "",
                raw_response=http_res,
            )
        if utils.match_response(http_res, ["4XX", "5XX"], "*"):
            raise errors.SDKError(
                "API error occurred", http_res.status_code, http_res.text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        raise errors.SDKError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res.text,
            http_res,
        )

    def create_dataset(
        self,
        *,
        request: Union[
            components.CreateDatasetRequest, components.CreateDatasetRequestTypedDict
        ],
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
    ) -> operations.CreateDatasetResponse:
        r"""Create a dataset

        :param request: The request object to send.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url

        if not isinstance(request, BaseModel):
            request = utils.unmarshal(request, components.CreateDatasetRequest)
        request = cast(components.CreateDatasetRequest, request)

        req = self.build_request(
            method="POST",
            path="/datasets",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request, False, False, "json", components.CreateDatasetRequest
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                operation_id="createDataset",
                oauth2_scopes=[],
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["4XX", "5XX"],
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "200", "application/json"):
            return operations.CreateDatasetResponse(
                object=utils.unmarshal_json(
                    http_res.text, Optional[operations.CreateDatasetResponseBody]
                ),
                status_code=http_res.status_code,
                content_type=http_res.headers.get("Content-Type") or "",
                raw_response=http_res,
            )
        if utils.match_response(http_res, ["4XX", "5XX"], "*"):
            raise errors.SDKError(
                "API error occurred", http_res.status_code, http_res.text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        raise errors.SDKError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res.text,
            http_res,
        )

    async def create_dataset_async(
        self,
        *,
        request: Union[
            components.CreateDatasetRequest, components.CreateDatasetRequestTypedDict
        ],
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
    ) -> operations.CreateDatasetResponse:
        r"""Create a dataset

        :param request: The request object to send.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url

        if not isinstance(request, BaseModel):
            request = utils.unmarshal(request, components.CreateDatasetRequest)
        request = cast(components.CreateDatasetRequest, request)

        req = self.build_request_async(
            method="POST",
            path="/datasets",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request, False, False, "json", components.CreateDatasetRequest
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                operation_id="createDataset",
                oauth2_scopes=[],
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["4XX", "5XX"],
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "200", "application/json"):
            return operations.CreateDatasetResponse(
                object=utils.unmarshal_json(
                    http_res.text, Optional[operations.CreateDatasetResponseBody]
                ),
                status_code=http_res.status_code,
                content_type=http_res.headers.get("Content-Type") or "",
                raw_response=http_res,
            )
        if utils.match_response(http_res, ["4XX", "5XX"], "*"):
            raise errors.SDKError(
                "API error occurred", http_res.status_code, http_res.text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        raise errors.SDKError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res.text,
            http_res,
        )

    def update_dataset(
        self,
        *,
        request: Union[components.DatasetUpdate, components.DatasetUpdateTypedDict],
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
    ) -> operations.UpdateDatasetResponse:
        r"""Update a dataset

        :param request: The request object to send.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url

        if not isinstance(request, BaseModel):
            request = utils.unmarshal(request, components.DatasetUpdate)
        request = cast(components.DatasetUpdate, request)

        req = self.build_request(
            method="PUT",
            path="/datasets",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="*/*",
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request, False, False, "json", components.DatasetUpdate
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                operation_id="updateDataset",
                oauth2_scopes=[],
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["4XX", "5XX"],
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "200", "*"):
            return operations.UpdateDatasetResponse(
                status_code=http_res.status_code,
                content_type=http_res.headers.get("Content-Type") or "",
                raw_response=http_res,
            )
        if utils.match_response(http_res, ["4XX", "5XX"], "*"):
            raise errors.SDKError(
                "API error occurred", http_res.status_code, http_res.text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        raise errors.SDKError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res.text,
            http_res,
        )

    async def update_dataset_async(
        self,
        *,
        request: Union[components.DatasetUpdate, components.DatasetUpdateTypedDict],
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
    ) -> operations.UpdateDatasetResponse:
        r"""Update a dataset

        :param request: The request object to send.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url

        if not isinstance(request, BaseModel):
            request = utils.unmarshal(request, components.DatasetUpdate)
        request = cast(components.DatasetUpdate, request)

        req = self.build_request_async(
            method="PUT",
            path="/datasets",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="*/*",
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request, False, False, "json", components.DatasetUpdate
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                operation_id="updateDataset",
                oauth2_scopes=[],
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["4XX", "5XX"],
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "200", "*"):
            return operations.UpdateDatasetResponse(
                status_code=http_res.status_code,
                content_type=http_res.headers.get("Content-Type") or "",
                raw_response=http_res,
            )
        if utils.match_response(http_res, ["4XX", "5XX"], "*"):
            raise errors.SDKError(
                "API error occurred", http_res.status_code, http_res.text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        raise errors.SDKError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res.text,
            http_res,
        )

    def delete_dataset(
        self,
        *,
        dataset_id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
    ) -> operations.DeleteDatasetResponse:
        r"""Delete a dataset

        :param dataset_id: The unique identifier of the dataset to be deleted like `663876ec4611c47f4970f0c3`
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url

        request = operations.DeleteDatasetRequest(
            dataset_id=dataset_id,
        )

        req = self.build_request(
            method="DELETE",
            path="/datasets",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="*/*",
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                operation_id="deleteDataset",
                oauth2_scopes=[],
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["4XX", "5XX"],
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "200", "*"):
            return operations.DeleteDatasetResponse(
                status_code=http_res.status_code,
                content_type=http_res.headers.get("Content-Type") or "",
                raw_response=http_res,
            )
        if utils.match_response(http_res, ["4XX", "5XX"], "*"):
            raise errors.SDKError(
                "API error occurred", http_res.status_code, http_res.text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        raise errors.SDKError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res.text,
            http_res,
        )

    async def delete_dataset_async(
        self,
        *,
        dataset_id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
    ) -> operations.DeleteDatasetResponse:
        r"""Delete a dataset

        :param dataset_id: The unique identifier of the dataset to be deleted like `663876ec4611c47f4970f0c3`
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url

        request = operations.DeleteDatasetRequest(
            dataset_id=dataset_id,
        )

        req = self.build_request_async(
            method="DELETE",
            path="/datasets",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="*/*",
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                operation_id="deleteDataset",
                oauth2_scopes=[],
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["4XX", "5XX"],
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "200", "*"):
            return operations.DeleteDatasetResponse(
                status_code=http_res.status_code,
                content_type=http_res.headers.get("Content-Type") or "",
                raw_response=http_res,
            )
        if utils.match_response(http_res, ["4XX", "5XX"], "*"):
            raise errors.SDKError(
                "API error occurred", http_res.status_code, http_res.text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        raise errors.SDKError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res.text,
            http_res,
        )

    def add_datapoints(
        self,
        *,
        dataset_id: str,
        request_body: Union[
            operations.AddDatapointsRequestBody,
            operations.AddDatapointsRequestBodyTypedDict,
        ],
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
    ) -> operations.AddDatapointsResponse:
        r"""Add datapoints to a dataset

        :param dataset_id: The unique identifier of the dataset to add datapoints to like  `663876ec4611c47f4970f0c3`
        :param request_body:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url

        request = operations.AddDatapointsRequest(
            dataset_id=dataset_id,
            request_body=utils.get_pydantic_model(
                request_body, operations.AddDatapointsRequestBody
            ),
        )

        req = self.build_request(
            method="POST",
            path="/datasets/{dataset_id}/datapoints",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.request_body,
                False,
                False,
                "json",
                operations.AddDatapointsRequestBody,
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                operation_id="addDatapoints",
                oauth2_scopes=[],
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["4XX", "5XX"],
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "200", "application/json"):
            return operations.AddDatapointsResponse(
                object=utils.unmarshal_json(
                    http_res.text, Optional[operations.AddDatapointsResponseBody]
                ),
                status_code=http_res.status_code,
                content_type=http_res.headers.get("Content-Type") or "",
                raw_response=http_res,
            )
        if utils.match_response(http_res, ["4XX", "5XX"], "*"):
            raise errors.SDKError(
                "API error occurred", http_res.status_code, http_res.text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        raise errors.SDKError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res.text,
            http_res,
        )

    async def add_datapoints_async(
        self,
        *,
        dataset_id: str,
        request_body: Union[
            operations.AddDatapointsRequestBody,
            operations.AddDatapointsRequestBodyTypedDict,
        ],
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
    ) -> operations.AddDatapointsResponse:
        r"""Add datapoints to a dataset

        :param dataset_id: The unique identifier of the dataset to add datapoints to like  `663876ec4611c47f4970f0c3`
        :param request_body:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url

        request = operations.AddDatapointsRequest(
            dataset_id=dataset_id,
            request_body=utils.get_pydantic_model(
                request_body, operations.AddDatapointsRequestBody
            ),
        )

        req = self.build_request_async(
            method="POST",
            path="/datasets/{dataset_id}/datapoints",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.request_body,
                False,
                False,
                "json",
                operations.AddDatapointsRequestBody,
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                operation_id="addDatapoints",
                oauth2_scopes=[],
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["4XX", "5XX"],
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "200", "application/json"):
            return operations.AddDatapointsResponse(
                object=utils.unmarshal_json(
                    http_res.text, Optional[operations.AddDatapointsResponseBody]
                ),
                status_code=http_res.status_code,
                content_type=http_res.headers.get("Content-Type") or "",
                raw_response=http_res,
            )
        if utils.match_response(http_res, ["4XX", "5XX"], "*"):
            raise errors.SDKError(
                "API error occurred", http_res.status_code, http_res.text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        raise errors.SDKError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res.text,
            http_res,
        )



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .evaluationrun import EvaluationRun, EvaluationRunTypedDict
from honeyhive.types import BaseModel
from typing import Optional, TypedDict
from typing_extensions import NotRequired


class GetRunResponseTypedDict(TypedDict):
    evaluation: NotRequired[EvaluationRunTypedDict]


class GetRunResponse(BaseModel):
    evaluation: Optional[EvaluationRun] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from honeyhive.models.components import tool as components_tool
from honeyhive.types import BaseModel
import httpx
from typing import List, Optional, TypedDict
from typing_extensions import NotRequired


class GetToolsResponseTypedDict(TypedDict):
    content_type: str
    r"""HTTP response content type for this operation"""
    status_code: int
    r"""HTTP response status code for this operation"""
    raw_response: httpx.Response
    r"""Raw HTTP response; suitable for custom response parsing"""
    tools: NotRequired[List[components_tool.ToolTypedDict]]
    r"""Successfully retrieved the list of tools"""


class GetToolsResponse(BaseModel):
    content_type: str
    r"""HTTP response content type for this operation"""

    status_code: int
    r"""HTTP response status code for this operation"""

    raw_response: httpx.Response
    r"""Raw HTTP response; suitable for custom response parsing"""

    tools: Optional[List[components_tool.Tool]] = None
    r"""Successfully retrieved the list of tools"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from honeyhive.types import BaseModel
from typing import Any, Dict, Optional, TypedDict
from typing_extensions import NotRequired


class UpdateToolRequestTypedDict(TypedDict):
    id: str
    name: str
    parameters: Dict[str, Any]
    description: NotRequired[str]


class UpdateToolRequest(BaseModel):
    id: str

    name: str

    parameters: Dict[str, Any]

    description: Optional[str] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from honeyhive.models.components import createmodelevent as components_createmodelevent
from honeyhive.types import BaseModel
import httpx
from typing import Optional, TypedDict
from typing_extensions import NotRequired


class CreateModelEventRequestBodyTypedDict(TypedDict):
    model_event: NotRequired[components_createmodelevent.CreateModelEventTypedDict]


class CreateModelEventRequestBody(BaseModel):
    model_event: Optional[components_createmodelevent.CreateModelEvent] = None


class CreateModelEventResponseBodyTypedDict(TypedDict):
    r"""Model event created"""

    event_id: NotRequired[str]
    success: NotRequired[bool]


class CreateModelEventResponseBody(BaseModel):
    r"""Model event created"""

    event_id: Optional[str] = None

    success: Optional[bool] = None


class CreateModelEventResponseTypedDict(TypedDict):
    content_type: str
    r"""HTTP response content type for this operation"""
    status_code: int
    r"""HTTP response status code for this operation"""
    raw_response: httpx.Response
    r"""Raw HTTP response; suitable for custom response parsing"""
    object: NotRequired[CreateModelEventResponseBodyTypedDict]
    r"""Model event created"""


class CreateModelEventResponse(BaseModel):
    content_type: str
    r"""HTTP response content type for this operation"""

    status_code: int
    r"""HTTP response status code for this operation"""

    raw_response: httpx.Response
    r"""Raw HTTP response; suitable for custom response parsing"""

    object: Optional[CreateModelEventResponseBody] = None
    r"""Model event created"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from honeyhive.types import BaseModel
from typing import Optional, TypedDict
from typing_extensions import NotRequired


class DeleteRunResponseTypedDict(TypedDict):
    id: NotRequired[str]
    deleted: NotRequired[bool]


class DeleteRunResponse(BaseModel):
    id: Optional[str] = None

    deleted: Optional[bool] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum
from honeyhive.types import BaseModel
from typing import Any, Dict, Optional, TypedDict
from typing_extensions import NotRequired


class CreateToolRequestType(str, Enum):
    FUNCTION = "function"
    TOOL = "tool"


class CreateToolRequestTypedDict(TypedDict):
    task: str
    r"""Name of the project associated with this tool"""
    name: str
    parameters: Dict[str, Any]
    r"""These can be function call params or plugin call params"""
    type: CreateToolRequestType
    description: NotRequired[str]


class CreateToolRequest(BaseModel):
    task: str
    r"""Name of the project associated with this tool"""

    name: str

    parameters: Dict[str, Any]
    r"""These can be function call params or plugin call params"""

    type: CreateToolRequestType

    description: Optional[str] = None



import uuid
from traceback import print_exc
import os
import sys
import threading
import io
from contextlib import redirect_stdout
import subprocess

# from honeyhive.utils.telemetry import Telemetry
from honeyhive.utils.baggage_dict import BaggageDict
from honeyhive.models import operations, components, errors
from honeyhive.sdk import HoneyHive

from traceloop.sdk import Traceloop
from traceloop.sdk.tracing.tracing import TracerWrapper

from opentelemetry import context, baggage
from opentelemetry.context import Context
from opentelemetry.sdk.metrics.export import ConsoleMetricExporter
from opentelemetry.propagators.composite import CompositePropagator
from opentelemetry.trace.propagation.tracecontext import TraceContextTextMapPropagator
from opentelemetry.baggage.propagation import W3CBaggagePropagator

DEFAULT_API_URL = "https://api.honeyhive.ai"

class HoneyHiveTracer:
    
    # static variables
    verbose = False
    _is_traceloop_initialized = False
    api_key = None
    is_evaluation = False
    server_url = None
    _flush_lock = threading.RLock()

    def __init__(
        self,
        api_key=None,
        project=None,
        session_name=None,
        source=None,
        server_url=None,
        session_id=None,
        disable_http_tracing=False,
        disable_batch=False,
        verbose=False,
        inputs=None,
        is_evaluation=False,
        run_id=None,
        dataset_id=None,
        datapoint_id=None,
        link_carrier=None
    ):
        
        # if HoneyHiveTracer is already initialized, get association properties from the context
        ctx: Context = context.get_current()
        association_properties = ctx.get('association_properties') if ctx is not None else None
        if association_properties is not None:
            # Unpack association properties by name
            session_id = association_properties.get('session_id')
            project = association_properties.get('project')
            source = association_properties.get('source')
            disable_http_tracing = association_properties.get('disable_http_tracing') or False
            run_id = association_properties.get('run_id')
            dataset_id = association_properties.get('dataset_id')
            datapoint_id = association_properties.get('datapoint_id')

        try:
            # api_key
            if HoneyHiveTracer.api_key is None:
                if api_key is None:
                    # get and validate api key from os env
                    env_api_key = os.getenv("HH_API_KEY")
                    if not HoneyHiveTracer._validate_api_key(env_api_key):
                        raise Exception("api_key must be specified or set in environment variable HH_API_KEY.")
                    api_key = env_api_key
                else:
                    # validate user-provided api key
                    if not HoneyHiveTracer._validate_api_key(api_key):
                        raise Exception("api_key must be a string.")
                    
                # set api key
                HoneyHiveTracer.api_key = api_key
            
            # server_url
            if HoneyHiveTracer.server_url is None:
                if server_url is None:
                    # get server url from os env with default
                    env_server_url = os.getenv("HH_API_URL", DEFAULT_API_URL)
                    if not HoneyHiveTracer._validate_server_url(env_server_url):
                        raise Exception("Invalid server URL in environment variable HH_API_URL.")
                    server_url = env_server_url
                else:
                    # validate user-provided server url
                    if not HoneyHiveTracer._validate_server_url(server_url):
                        raise Exception("server_url must be a valid URL string.")
                # set server url
                HoneyHiveTracer.server_url = server_url
            
            # project
            if project is None:
                project = os.getenv("HH_PROJECT")
                if project is None:
                    raise Exception("project must be specified or set in environment variable HH_PROJECT.")
            
            # session_name
            if session_name is None:
                try:
                    session_name = os.path.basename(sys.argv[0])
                except Exception as e:
                    if HoneyHiveTracer.verbose:
                        print(f"Error setting session_name: {e}")
                    session_name = "unknown"

            # source
            if source is None:
                source = os.getenv("HH_SOURCE", "dev")
            
            # verbose
            HoneyHiveTracer.verbose = verbose
            
            # TODO: migrate to log-based session initialization
            # self.session_id = str(uuid.uuid4()).upper()
            if session_id is None:
                # Add git information to metadata only if successful
                git_info = HoneyHiveTracer._get_git_info()
                metadata = git_info if "error" not in git_info else None
                
                # Store necessary parameters as instance variables
                self.session_name = session_name
                self.inputs = inputs
                self.metadata = metadata
                self.project = project
                self.source = source
                
                # Start the session and get session_id
                self.session_start()
            else:
                # Validate that session_id is a valid UUID
                try:
                    uuid.UUID(session_id)
                    self.session_id = session_id.lower()
                    self.project = project
                    self.source = source
                except (ValueError, AttributeError, TypeError):
                    raise errors.SDKError("session_id must be a valid UUID string.")

            # Initialize baggage with all parameters
            self.baggage = BaggageDict().update({
                "session_id": self.session_id,
                "project": project,
                "source": source,
                "disable_http_tracing": str(disable_http_tracing).lower(),
            })

            # Add evaluation specific properties if needed
            if is_evaluation:
                self.baggage.update({
                    "run_id": run_id,
                    "dataset_id": dataset_id,
                    "datapoint_id": datapoint_id,
                })

            # Initialize the Composite Propagator
            HoneyHiveTracer.propagator = CompositePropagator(
                propagators=[
                    TraceContextTextMapPropagator(),
                    W3CBaggagePropagator()
                ]
            )

            # instrument tracer with lock
            with threading.Lock():
                # Initialize Traceloop with CompositePropagator
                if not HoneyHiveTracer._is_traceloop_initialized:
                    traceloop_args = {
                        "api_endpoint": f"{HoneyHiveTracer.server_url}/opentelemetry",
                        "api_key": HoneyHiveTracer.api_key,
                        "metrics_exporter": ConsoleMetricExporter(out=open(os.devnull, "w")),
                        "disable_batch": disable_batch,
                        "propagator": HoneyHiveTracer.propagator
                    }

                    # Only redirect stdout if verbose is False
                    if not HoneyHiveTracer.verbose:
                        with redirect_stdout(io.StringIO()):
                            Traceloop.init(**traceloop_args)
                    else:
                        Traceloop.init(**traceloop_args)
                    
                    # Print initialization message in orange color (works in both bash and Windows)
                    if not HoneyHiveTracer.is_evaluation:
                        print("\033[38;5;208mHoneyHive is initialized\033[0m")
                    HoneyHiveTracer._is_traceloop_initialized = True
                    HoneyHiveTracer.is_evaluation = is_evaluation
                # Telemetry().capture("tracer_init", {"hhai_session_id": self.session_id, "hhai_project": project})

            # link_carrier
            if link_carrier is not None:
                self.link(link_carrier)
            else:
                # attach baggage to the current context
                ctx = context.get_current() # deep copy of the current context
                ctx = self.baggage.set_all_baggage(ctx)
                context.attach(ctx)
            
            # traceloop sets "association_properties" in the context
            # however it is not propagated since it doesn't follow the W3C spec for Baggage
            # since traceloop stamps "association_properties" from the context into every span when it starts, 
            # we must attach the baggage in traceloop format as well
            # Traceloop.set_baggage_properties(self.baggage)
            Traceloop.set_association_properties(self.baggage)
            
            # ------------------------------------------------------------
            # TODO: log-based session initialization
            # ------------------------------------------------------------
            # save the init metadata
            # self._init_metadata = {
            #     "project": project,
            #     "session_name": session_name,
            #     "source": source,
            #     "server_url": server_url,
            #     "verbose": verbose,
            #     "disable_batch": disable_batch,
            #     "link_carrier_provided": link_carrier is not None,
            #     "instrumentation_id": HoneyHiveTracer.instrumentation_id,
            # }
            
            # # log the session initialization
            # @trace
            # def __session_init():
            #     enrich_span(metadata={
            #         '_init_metadata': self._init_metadata
            #     })
            # __session_init()
            # ------------------------------------------------------------
        except errors.SDKError as e:
            # Raise SDK exceptions as-is but without traceback
            print(f"\033[91mHoneyHive SDK Error: {str(e)}\033[0m")
        except:
            # Log other exceptions if verbose is enabled
            if HoneyHiveTracer.verbose:
                print_exc()
            else:
                pass


    # TODO: remove this, legacy DX
    @staticmethod
    def init(*args, **kwargs):
        return HoneyHiveTracer(*args, **kwargs)
    
    @staticmethod
    def _validate_api_key(api_key):
        return api_key and type(api_key) == str
    
    @staticmethod
    def _validate_server_url(server_url):
        return server_url and type(server_url) == str
    
    def session_start(self) -> str:
        """Start a session using the tracer's parameters"""
        self.session_id = HoneyHiveTracer.__start_session(
            HoneyHiveTracer.api_key, 
            self.project, 
            self.session_name, 
            self.source, 
            HoneyHiveTracer.server_url, 
            self.inputs, 
            self.metadata
        )
        return self.session_id
    
    @staticmethod
    def _get_git_info():
        try:
            # Check if telemetry is disabled
            telemetry_disabled = os.getenv("HONEYHIVE_TELEMETRY", "true").lower() in ["false", "0", "f", "no", "n"]
            if telemetry_disabled:
                if HoneyHiveTracer.verbose:
                    print("Telemetry disabled. Skipping git information collection.")
                return {"error": "Telemetry disabled"}
                
            cwd = os.getcwd()
            
            # First check if this is a git repository
            is_git_repo = subprocess.run(
                ["git", "rev-parse", "--is-inside-work-tree"],
                cwd=cwd, capture_output=True, text=True, check=False
            )
            
            # If not a git repo, return early with an error
            if is_git_repo.returncode != 0:
                if HoneyHiveTracer.verbose:
                    print("Not a git repository. Skipping git information collection.")
                return {"error": "Not a git repository"}
                
            commit_hash = subprocess.run(
                ["git", "rev-parse", "HEAD"],
                cwd=cwd, capture_output=True, text=True, check=True
            ).stdout.strip()

            branch = subprocess.run(
                ["git", "rev-parse", "--abbrev-ref", "HEAD"],
                cwd=cwd, capture_output=True, text=True, check=True
            ).stdout.strip()

            repo_url = subprocess.run(
                ["git", "config", "--get", "remote.origin.url"],
                cwd=cwd, capture_output=True, text=True, check=True
            ).stdout.strip().rstrip('.git')

            commit_link = f"{repo_url}/commit/{commit_hash}" if "github.com" in repo_url else repo_url

            status = subprocess.run(
                ["git", "status", "--porcelain"],
                cwd=cwd, capture_output=True, text=True, check=True
            ).stdout.strip()

            has_uncommitted_changes = bool(status)

            repo_root = subprocess.run(
                ["git", "rev-parse", "--show-toplevel"],
                cwd=cwd, capture_output=True, text=True, check=True
            ).stdout.strip()
            
            # Get relative path of the main module
            main_module = sys.modules.get('__main__')
            relative_path = None
            if main_module and hasattr(main_module, '__file__'):
                absolute_path = os.path.abspath(main_module.__file__)
                relative_path = os.path.relpath(absolute_path, repo_root)

            return {
                "commit_hash": commit_hash,
                "branch": branch,
                "repo_url": repo_url,
                "commit_link": commit_link,
                "uncommitted_changes": has_uncommitted_changes,
                "relative_path": relative_path
            }
        except subprocess.CalledProcessError:
            if HoneyHiveTracer.verbose:
                print("Failed to retrieve Git info. Is this a valid repo?")
            return {"error": "Failed to retrieve Git info. Is this a valid repo?"}
        except FileNotFoundError:
            if HoneyHiveTracer.verbose:
                print("Git is not installed or not in PATH.")
            return {"error": "Git is not installed or not in PATH."}
        except Exception as e:
            if HoneyHiveTracer.verbose:
                print(f"Error getting git info: {e}")
            return {"error": f"Error getting git info: {e}"}
    
    @staticmethod
    def __start_session(api_key, project, session_name, source, server_url, inputs=None, metadata=None):
        sdk = HoneyHive(bearer_auth=api_key, server_url=server_url)
        res = sdk.session.start_session(
            request=operations.StartSessionRequestBody(
                session=components.SessionStartRequest(
                    project=project,
                    session_name=session_name,
                    source=source,
                    inputs=inputs or {},
                    metadata=metadata or {}
                )
            )
        )
        assert res.status_code == 200, f"Failed to start session: {res.raw_response.text}"
        assert res.object.session_id is not None, "Failure initializing session"
        return res.object.session_id
    
    def _sanitize_carrier(carrier, getter):
        # check for baggage in the headers, potentially re-cased
        _propagation_carrier = {}
        for key in ['baggage', 'traceparent']:
            carrier_value = \
                getter.get(carrier, key.lower()) or \
                getter.get(carrier, key.capitalize()) or \
                getter.get(carrier, key.upper())
            if carrier_value is not None:
                _propagation_carrier[key] = [carrier_value]
        return _propagation_carrier
    
    def link(self, carrier={}, getter=BaggageDict.DefaultGetter):
        ctx = context.get_current() # deep copy of the current context
        
        # extract baggage from the carrier
        carrier = HoneyHiveTracer._sanitize_carrier(carrier, getter)
        ctx = HoneyHiveTracer.propagator.extract(carrier, ctx, getter=getter)
        
        # attach the baggage to the current context
        token = context.attach(ctx)
        
        # current context should now have baggage and span context from the carrier
        # it has been fully linked to the parent context
        
        # update the Traceloop baggage in the current context
        # this will be stamped on every span in this context
        bags = self.baggage.get_all_baggage()
        Traceloop.set_association_properties(bags)
        
        return token
    
    def unlink(self, token):
        # included for completeness, but not necessary
        context.detach(token)
        bags = self.baggage.get_all_baggage()
        Traceloop.set_association_properties(bags)
    
    def inject(self, carrier={}, setter=BaggageDict.DefaultSetter):
        # inject current trace and baggage context into the carrier
        HoneyHiveTracer.propagator.inject(carrier, None, setter)
        return carrier

    @staticmethod
    def flush():
        """
        Flush the tracer.
        Thread-safe and coroutine-safe - can be called from both threaded and async contexts.
        
        In async context, call with:
          await asyncio.to_thread(HoneyHiveTracer.flush)
        """
        if not HoneyHiveTracer._is_traceloop_initialized:
            print("\033[91mCould not flush: HoneyHiveTracer not initialized successfully\033[0m")
            return
        
        # Try to acquire the lock without blocking
        # If already locked, return immediately instead of waiting
        if not HoneyHiveTracer._flush_lock.acquire(blocking=False):
            # Lock already taken, another flush is in progress
            return
        
        try:
            TracerWrapper().flush()
        finally:
            # Always release the lock
            HoneyHiveTracer._flush_lock.release()

    def enrich_session(
        self,
        session_id=None,
        metadata=None, 
        feedback=None, 
        metrics=None, 
        config=None, 
        inputs=None, 
        outputs=None, 
        user_properties=None
    ):
        # TODO: migrate to log-based session enrichments
        # @trace
        # def __enrich_session():
        #     _enrichments = {}
        #     if metadata is not None:
        #         _enrichments["metadata"] = metadata
        #     if feedback is not None:
        #         _enrichments["feedback"] = feedback
        #     if metrics is not None:
        #         _enrichments["metrics"] = metrics
        #     if config is not None:
        #         _enrichments["config"] = config
        #     if inputs is not None:
        #         _enrichments["inputs"] = inputs
        #     if outputs is not None:
        #         _enrichments["outputs"] = outputs
        #     if user_properties is not None:
        #         _enrichments["user_properties"] = user_properties
        #     enrich_span(metadata={
        #         '_enrichments': _enrichments,
        #         '_init_metadata': self._init_metadata
        #     })
        # __enrich_session()
        if not HoneyHiveTracer._is_traceloop_initialized:
            print("\033[91mCould not enrich session: HoneyHiveTracer not initialized successfully\033[0m")
            return
        
        session_id = session_id or self.session_id
        try:
            sdk = HoneyHive(bearer_auth=HoneyHiveTracer.api_key, server_url=HoneyHiveTracer.server_url)
            update_request = operations.UpdateEventRequestBody(event_id=session_id)
            if feedback is not None:
                update_request.feedback = feedback
            if metrics is not None:
                update_request.metrics = metrics
            if metadata is not None:
                update_request.metadata = metadata
            if config is not None:
                update_request.config = config
            if inputs is not None:
                print('inputs are not supported in enrich_session') # TODO: add support for inputs (type change)
            if outputs is not None:
                update_request.outputs = outputs
            if user_properties is not None:
                update_request.user_properties = user_properties
            response: operations.UpdateEventResponse = sdk.events.update_event(request=update_request)
            if response.status_code != 200:
                raise Exception(f"Failed to enrich session: {response.raw_response.text}")
        except:
            if HoneyHiveTracer.verbose:
                print_exc()
            else:
                pass


def enrich_session(
    session_id=None,
    metadata=None,
    feedback=None,
    metrics=None,
    config=None,
    inputs=None,
    outputs=None,
    user_properties=None
):
    print()
    if not HoneyHiveTracer._is_traceloop_initialized:
        print("\033[91mCould not enrich session: HoneyHiveTracer not initialized successfully\033[0m")
        return
    try:
        sdk = HoneyHive(bearer_auth=HoneyHiveTracer.api_key, server_url=HoneyHiveTracer.server_url)
        if session_id is None:
            ctx: Context = context.get_current()
            association_properties = ctx.get('association_properties') if ctx is not None else None
            if association_properties is not None:
                session_id = association_properties.get('session_id')
            if session_id is None:
                raise Exception("Please initialize HoneyHiveTracer before calling enrich_session")
            
        update_request = operations.UpdateEventRequestBody(event_id=session_id.lower())
        if feedback is not None:
            update_request.feedback = feedback
        if metrics is not None:
            update_request.metrics = metrics
        if metadata is not None:
            update_request.metadata = metadata
        if config is not None:
            update_request.config = config
        if inputs is not None:
            print('inputs are not supported in enrich_session') # TODO: add support for inputs (type change)
        if outputs is not None:
            update_request.outputs = outputs
        if user_properties is not None:
            update_request.user_properties = user_properties
        response: operations.UpdateEventResponse = sdk.events.update_event(request=update_request)
        if response.status_code != 200:
            raise Exception(f"Failed to enrich session: {response.raw_response.text}")
    except:
        if HoneyHiveTracer.verbose:
            print_exc()
        else:
            pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from honeyhive.models.components import (
    putconfigurationrequest as components_putconfigurationrequest,
)
from honeyhive.types import BaseModel
from honeyhive.utils import FieldMetadata, PathParamMetadata, RequestMetadata
import httpx
from typing import TypedDict
from typing_extensions import Annotated


class UpdateConfigurationRequestTypedDict(TypedDict):
    id: str
    r"""Configuration ID like `6638187d505c6812e4043f24`"""
    put_configuration_request: (
        components_putconfigurationrequest.PutConfigurationRequestTypedDict
    )


class UpdateConfigurationRequest(BaseModel):
    id: Annotated[
        str, FieldMetadata(path=PathParamMetadata(style="simple", explode=False))
    ]
    r"""Configuration ID like `6638187d505c6812e4043f24`"""

    put_configuration_request: Annotated[
        components_putconfigurationrequest.PutConfigurationRequest,
        FieldMetadata(request=RequestMetadata(media_type="application/json")),
    ]


class UpdateConfigurationResponseTypedDict(TypedDict):
    content_type: str
    r"""HTTP response content type for this operation"""
    status_code: int
    r"""HTTP response status code for this operation"""
    raw_response: httpx.Response
    r"""Raw HTTP response; suitable for custom response parsing"""


class UpdateConfigurationResponse(BaseModel):
    content_type: str
    r"""HTTP response content type for this operation"""

    status_code: int
    r"""HTTP response status code for this operation"""

    raw_response: httpx.Response
    r"""Raw HTTP response; suitable for custom response parsing"""
